/**
 * Copyright (c) Microsoft Corporation 2006-2008. All Rights Reserved.
 * Information Contained Herein is Proprietary and Confidential.
 */
package com.microsoft.jars.VoiceSearch;
import java.util.Vector;

/**
 * The following class represents a SNS phrase element or an element alternate.
 */
public class SnsPhraseElement {

    /** Text associated with this SNS phrase element. */
    private String text;

    /**
     * Flag specifying whether this element consumes any preceding space
     * character.
     */
    private boolean mustConsumeLeadingSpace;

    /** Flag specifying whether this element is followed by a space character. */
    private boolean mustAddTrailingSpace;

    /** Alternates for this SNS phrase element. */
    private Vector<SnsPhraseElement> alternates;

    /**
     * Allocates a new <code>SnsPhraseElement</code> with the given text and
     * flags.
     * 
     * @param text
     *                the text of this SNS phrase element.
     * @param mustAddTrailingSpace
     *                the flag to specify whether this element is followed by a
     *                space character.
     * @param mustConsumeLeadingSpace
     *                the flag to specify whether this element consumes any
     *                preceding space character.
     */
    public SnsPhraseElement(String text, boolean mustAddTrailingSpace,
            boolean mustConsumeLeadingSpace) {
        this.text = text;
        this.mustConsumeLeadingSpace = mustConsumeLeadingSpace;
        this.mustAddTrailingSpace = mustAddTrailingSpace;
    }

    /**
     * Clones a <code>SnsPhraseElement</code>.
     * 
     * @param element
     *                the element to clone.
     */
    public SnsPhraseElement(SnsPhraseElement element) {
        this.text = element.text;
        this.mustConsumeLeadingSpace = element.mustConsumeLeadingSpace;
        this.mustAddTrailingSpace = element.mustAddTrailingSpace;
        this.alternates = element.alternates;
    }

    /**
     * Gets the text associated with this element.
     * 
     * @return the text associated with this element.
     */
    public String getText() {
        return text;
    }

    /**
     * Gets the flag specifying whether this element consumes any preceding
     * space character.
     * 
     * @return <code>true</code> if this element consumes any preceding space
     *         character; <code>false</code> otherwise.
     */
    public boolean mustConsumeLeadingSpace() {
        return mustConsumeLeadingSpace;
    }

    /**
     * Gets the flag specifying whether this element is followed by a space
     * character.
     * 
     * @return <code>true</code> if this element is followed by a space
     *         character; <code>false</code> otherwise.
     */
    public boolean mustAddTrailingSpace() {
        return mustAddTrailingSpace;
    }

    /**
     * Adds an alternate to this element.
     * 
     * @param alternate
     *                the alternate to add.
     */
    public void addAlternate(SnsPhraseElement alternate) {
        if (alternates == null) {
            alternates = new Vector<SnsPhraseElement>();
        }
        alternates.addElement(alternate);
    }
}
