/**
 * Copyright (c) Microsoft Corporation 2006-2008. All Rights Reserved.
 * Information Contained Herein is Proprietary and Confidential.
 */

package com.microsoft.jars.VoiceSearch;
import java.io.InputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import com.microsoft.mobileexperiences.bing.httpthreadpool.HttpResponseBase;



/**
 * The following class represents a SNS response.
 */
public class SnsResponse extends HttpResponseBase {

    /** Equal character separating property key and value. */
    private static final char PROPERTY_EQUAL_CHARACTER = '=';

    /** Mask to test addition of trailing space character. */
    private static final int ADD_TRAILING_SPACE_MASK = 0x80;

    /** Mask to test consumption of leading space character. */
    private static final int CONSUME_LEADING_SPACE_MASK = 0x40;

    /** Mask to get the length of an element text. */
    private static final int LENGTH_MASK = 0x3f;

    /** Version of this SNS response. */
    private byte version;

    /** Name of the response stream. */
    private String name;

    /** Properties associated with this SNS response. */
    private Hashtable<String, String> properties;

    /** Results included in this SNS response. */
    private Vector<SnsResult> results;

    /** Table containing elements which can be referenced by IDs. */
    private Vector<SnsPhraseElement> elementTable;

    /** String buffer helper. */
    private StringBuffer stringBuffer;
    
    private String resultBody;

    //private BusinessRequestData[] businessData;

    /**
     * Allocates a new <code>SnsResponse</code>.
     */
    public SnsResponse() {
    	super();
        properties = new Hashtable<String, String>();
        results = new Vector<SnsResult>();
        elementTable = new Vector<SnsPhraseElement>();
        stringBuffer = new StringBuffer();
        //businessData = null;
    }
    
    public final Vector<String> getPhases()
    {
    	Vector<String> results = new Vector<String>();
	
		for(int i = 0; i< getResults().size() ; i++)
		{
			//results.add(response.getResults().get(i).getName());
			for(int j = 0; j< getResults().get(i).getPhraseCount(); j++)
			{
				results.add(getResults().get(i).getPhrase(j).getText());
			}
		}							
		
    	return 	results;
    }
    
    public final String getResultBody()
    {
    	return resultBody;
    }

    /**
     * Resets all the data from this SNS response.
     */
    private void reset() {
        version = 0;
        name = null;
        properties.clear();
        results.removeAllElements();
        elementTable.removeAllElements();
        //businessData = null;
    }

    /**
     * Reads the SNS response from the input stream given as argument and
     * initializes this object.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    public void init(InputStream iStream) throws IOException {
        DataInputStream dis = new DataInputStream(iStream);
        
        reset();
        
        // read version
        version = dis.readByte();

        // read name
        name = readString(dis);

        // read properties
        readProperties(dis);

        // read results
        readResults(dis);   
        
        // read search results
        if (results.size() > 0)
        	readSearchResults(dis, iStream);

    }
    
    /**
     * Read the local search results.
     * @param dis data input stream
     * @param iStream the input stream associated with this http connection
     * @param httpConnection the http connection
     */
    private void readSearchResults(final DataInputStream dis, final InputStream iStream) 
    {
        // now already the voice recognition results.  The search results come in 
        // only when they're ready and so we read the search results in a thread so that
        // the app can use the voice recognition results immediately.
      
               
        byte[] buffer = new byte[4];
       
        SnsResult result = (SnsResult)(results.elementAt(0));
        String phraseName = null;
        
        try
        {
            phraseName = null;
            
            // each search result starts with the length of the result.  
            // The length = length of the name + the byte holding the length of the name + the length of the search 
            // result body which starts at "<s:Envelope xmlns:s="http://...".
            
            // the length is a 32-bit number
            // read the low word portion first
            int length_low_word = dis.readShort();
            length_low_word = VoiceSearchRequest.revertShort(length_low_word);    // reverse the byte order since the data came from a Window PC
            
            // read the high word portion
            int length_high_word = dis.readShort();
            length_high_word = VoiceSearchRequest.revertShort(length_high_word);  
            
            int length = length_high_word << 16 | length_low_word;
            
            // read 4 bytes of stuff - not sure what these bytes are.
            dis.read(buffer);
                                          
            // followed the length of the phrase and then the name of the phrase
            byte nameLength = dis.readByte();
            phraseName = readString(dis, nameLength);
            
            int index = result.getPhraseIndex(phraseName);
            if (index != -1)
            {
                // the name is followed by the search result body
                // need to adjust the length for the search result body(total length - length of the name - the byte holding the length of the name)
                length = length - nameLength -1;
                resultBody = readBigString(dis, length);
                //result.getPhrase(index).setRawSearchResult(resultBody);
            }
        } catch (Exception e)
        {
            // eat any error and we'll set an empty result to the phrase in the finally-block below.
        }                       
             
	}
        
    private String readBigString(DataInputStream dis, int length)
    throws IOException 
	{
	    byte[] buffer = new byte[length];
	    int start = 0;
	    int bytesToRead = length;
	    do
	    {
	        int bytesRead = dis.read(buffer, start, bytesToRead);
	        if (bytesRead == -1)
	        {
	            break;
	        }
	        bytesToRead -= bytesRead;
	        start += bytesRead;
	    } while (bytesToRead > 0);
	    
	    String s = new String(buffer, 0, length);
	    
	    return s;
	}



    /**
     * Reads the SNS response properties from the input stream given as
     * argument.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    private void readProperties(DataInputStream dis) throws IOException {
        byte b;
        while ((b = dis.readByte()) != 0) {

            // read property key
            stringBuffer.delete(0, stringBuffer.length());
            while (b != PROPERTY_EQUAL_CHARACTER) {
                stringBuffer.append((char) b);
                if ((b = dis.readByte()) == 0) {
                    break;
                }
            }
            String key = stringBuffer.toString().trim();

            // read property value
            stringBuffer.delete(0, stringBuffer.length());
            while ((b = dis.readByte()) != 0) {
                stringBuffer.append((char) b);
            }

            properties.put(key, stringBuffer.toString().trim());
        }
    }

    /**
     * Reads the results from the input stream given as argument. A result is
     * represented by <its name, element table, phrases, alternates>.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    private void readResults(DataInputStream dis) throws IOException {
        while (true) {
            String name = readString(dis);
            if (name == null) {
                return;
            }
            SnsResult result = new SnsResult(name);
            readElementTable(dis);
            readPhrases(dis, result);
            readAlternates(dis, result);
            results.addElement(result);
        }
    }

    /**
     * Reads an element table from the input stream given as argument. An
     * element table is a sequence of <first byte, text> and it is terminated by
     * a byte 0. byte_7 of the first byte specifies whether the element must be
     * followed by a space character and byte_6 specifies whether the element
     * consumes any preceding one. byte_0 to byte_5 represents the length of the
     * follogin text.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    private void readElementTable(DataInputStream dis) throws IOException {
        byte b;
        while ((b = dis.readByte()) != 0) {
            String name = readString(dis, b & LENGTH_MASK);
            boolean mustAddTrailingSpace = (b & ADD_TRAILING_SPACE_MASK) != 0;
            boolean mustConsumeLeadingSpace = (b & CONSUME_LEADING_SPACE_MASK) != 0;
            SnsPhraseElement element = new SnsPhraseElement(name,
                    mustAddTrailingSpace, mustConsumeLeadingSpace);
            elementTable.addElement(element);
        }
    }

    /**
     * Reads phrases from the input stream given as argument. A phrase is a
     * sequence of number IDs referencing element in the element table, and it
     * is terminated by a byte 0.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @param result
     *                the <code>SnsResult</code> which will contain the read
     *                phrases.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    private void readPhrases(DataInputStream dis, SnsResult result)
            throws IOException {
        byte b;
        int elementTableSize = elementTable.size();
        while ((b = dis.readByte()) != 0) {
            SnsPhrase phrase = new SnsPhrase();
            do {
                int id = b;

                // testing continuation bit
                if ((id & 0x80) != 0) {
                    id = ((id & 0x7f) << 8) | dis.readByte();
                }
                // index starts from 1 so we must substract 1 from it
                id--;
                if (id >= 0 && id < elementTableSize) {
                    SnsPhraseElement element = (SnsPhraseElement) elementTable
                            .elementAt(id);

                    // clone element as it may have alternates in this phrase
                    phrase.addElement(new SnsPhraseElement(element));
                }
            } while ((b = dis.readByte()) != 0);
            result.addPhrase(phrase);
        }
    }

    /**
     * Reads alternates from the input stream given as argument.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @param result
     *                the <code>SnsResult</code> which contains the read
     *                phrases.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    private void readAlternates(DataInputStream dis, SnsResult result)
            throws IOException {
        readElementTable(dis);
        int elementTableSize = elementTable.size();
        int phraseCount = result.getPhraseCount();
        for (int i = 0; i < phraseCount; i++) {
            SnsPhrase phrase = result.getPhrase(i);
            int elementCount = phrase.getElementCount();
            for (int j = 0; j < elementCount; j++) {
                SnsPhraseElement element = phrase.getElement(j);
                byte b;
                while ((b = dis.readByte()) != 0) {
                    do {
                        int index = b;

                        // testing continuation bit
                        if ((index & 0x80) != 0) {
                            index = ((index & 0x7f) << 8) | dis.readByte();
                        }
                        // index starts from 1 so we must substract 1 from it
                        index--;
                        if (index >= 0 && index < elementTableSize) {
                            SnsPhraseElement alternate = (SnsPhraseElement) elementTable
                                    .elementAt(index);
                            element.addAlternate(alternate);
                        }
                    } while ((b = dis.readByte()) != 0);
                }
            }
        }
    }

    /**
     * Reads business data from the inputstream given as argument and initialize
     * the business data array.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from where the business
     *                data are read.
     * @throws IOException
     *                 if an I/O error occurs.
     */
     /*
    private void readBusinessData(DataInputStream dis) throws IOException {
        int queryNumber = dis.readUnsignedByte();
        businessData = new BusinessRequestData[queryNumber];
        for (int i = 0; i < queryNumber; i++) {
            businessData[i] = BusinessRequest.readBusinessRequestData(dis);
        }
    }*/
    
   /**
     * Reads a <code>String</code> from the input stream given as argument,
     * byte by byte until byte 0 is read.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @return the <code>String</code> read from the input stream.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    private String readString(DataInputStream dis) throws IOException {
        byte b;
        stringBuffer.delete(0, stringBuffer.length());
        while ((b = dis.readByte()) != 0) {
            stringBuffer.append((char) b);
        }
        if (stringBuffer.length() == 0) {
            return null;
        }
        return stringBuffer.toString();
    }

    /**
     * Reads a <code>String</code> of the given length from the input stream
     * given as argument. The method does not stop when it encounters a byte 0.
     * 
     * @param dis
     *                the <code>DataInputStream</code> from which the SNS
     *                response is read.
     * @param length
     *                the length of the <code>String</code> which must be
     *                read.
     * @return the <code>String</code> read from the input stream, of the
     *         given length.
     * @throws IOException
     *                 if an I/O error occurs.
     */
    private String readString(DataInputStream dis, int length)
            throws IOException {
        stringBuffer.delete(0, stringBuffer.length());
        for (int i = length; i-- > 0;) {
            stringBuffer.append((char) dis.readByte());
        }
        return stringBuffer.toString();
    }

    /**
     * Gets the version of this SNS response.
     * 
     * @return the version of this SNS response.
     */
    public byte getVersion() {
        return version;
    }

    /**
     * Gets the name of this SNS response.
     * 
     * @return the name of this SNS response.
     */
    public String getName() {
        return name;
    }

    /**
     * Gets the properties of this SNS response.
     * 
     * @return the properties of this SNS response.
     */
    public Hashtable<String, String> getProperties() {
        return properties;
    }

    /**
     * Gets the results of this SNS response.
     * 
     * @return the properties of this SNS response.
     */
    public Vector<SnsResult> getResults() {
        return results;
    }

    /**
     * Gets the business data array.
     * 
     * @return the business data array.
     */
     /*
    public BusinessRequestData[] getBusinessData() {
        return businessData;
    }*/
}
