package com.microsoft.mobileexperiences.bing.httpthreadpool;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;
import java.util.concurrent.Future;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import android.util.Log;

public abstract class HttpRequest implements Runnable{

	public enum HttpMethod
	{
		HTTP_METHOD_GET, 
		HTTP_METHOD_POST
	}
	
	public enum Priority
	{
		HIGH,
		LOW
	}

	protected String url;
	private HttpMethod httpMethod;
	private Boolean gzipUploadStream = false;
	private int timeOut = 15000; //30 seconds
	private int readTimeout = 15000;
	
	private Future<?> future = null;
	private Hashtable<String, String> headers = new Hashtable<String, String>();
	/**
	 * @return the headers
	 */
	public Hashtable<String, String> getHeaders() {
		return headers;
	}
	
	public void setTimout(int connectionTimeout, int readTimeout){
		this.timeOut = connectionTimeout;
		this.readTimeout = readTimeout;
	}
	
	private HttpResponseBase httpResponse;
	protected IHttpResult httpCallback;
	private boolean httpChunkedStreamingMode = false;
	private int m_ChunkedStreamSize = 1024;
	private Priority priority;
	protected boolean cancelled = false;

	public HttpRequest(String url, HttpMethod httpMethod, Priority priority)
	{
		this.url = url;
		this.httpMethod = httpMethod;
		this.priority = priority;
	}
	
	public HttpRequest(String url, HttpMethod httpMethod)
	{
		this(url, httpMethod, Priority.LOW);
	}
	
	public String getUrl(){
		return this.url;
	}
	
	public void setUrl(String url)
	{
		this.url = url;
	}
	
	public void setCallback(IHttpResult httpCallback)
	{
		this.httpCallback = httpCallback;
		cancelled = false;
	}
	
	public void setFuture(Future<?> future) //used by thread pool class. 
	{
		this.future = future;
	}
	
	public void setChunkedStreamingMode(boolean mode)
	{
		this.httpChunkedStreamingMode = mode;
	}
	
	public void setChunkedStreamSize(int size)
	{
		if (size > 0)
			m_ChunkedStreamSize = size;
	}
	
	public void setReadTimeout(int timeout)
	{
		this.readTimeout = timeout;
	}
	
	public Future<?> getFuture()  
	{
		return future;
	}
	
	public HttpResponseBase getResponse()
	{
		return httpResponse;
	}
	
	public Priority getPriority()
	{
		return priority;
	}
	
	//default decode function. Could be overrided.
	protected HttpResponseBase decode(InputStream stream, Hashtable<String, String> headers)
	{
		HttpDefaultResponse response = new HttpDefaultResponse(stream, headers);
		return response;
	}

	public String encode() {return null;} //need to override it for POST request. 
	public abstract void setHttpHeaders(); //need to be override.
	public void streamContentOut(OutputStream outputStream) {}  //for voice search purpose.
	
	public void setTimeOut(int timeOut)
	{
		this.timeOut = timeOut;
	}

	public void setGZip(Boolean gzipped)
	{
		gzipUploadStream = gzipped;
	}

	public void addHttpHeader(String name, String value)
	{
		if (name != null && name.length() > 0 && value != null && value.length() > 0)
			headers.put(name, value);
	}
	
	public void cancel()
	{
		if(future != null) {
			cancelled = true;		
			future.cancel(true);
			future = null;
		}
	}
		
	@SuppressWarnings("static-access")
	public void run()  
	{		
		HttpURLConnection connection = null;
		InputStream inputStream = null;
		boolean cleanupErrorStream = false;
		long requestSentoutTimeStamp = System.currentTimeMillis();
		
		try {
			//#ifdef zh-CN
			//#if ((null != HttpThreadPool.activeNetworkInfo) &&
					//#((HttpThreadPool.activeNetworkInfo.contains("cmwap") ||
					//#HttpThreadPool.activeNetworkInfo.contains("ctwap") ||
					//#HttpThreadPool.activeNetworkInfo.contains("uniwap") ||
					//#HttpThreadPool.activeNetworkInfo.contains("3gwap"))))
			//#{
				//#URL tmpURL = new URL(url);
				//#String hostName = tmpURL.getHost();
				//#url = url.replace(hostName, "10.0.0.172");
				//#connection = (HttpURLConnection) new URL(url).openConnection();
				//#connection.setRequestProperty("X-Online-Host", hostName);
			//#}
			//##else
				//#connection = (HttpURLConnection) new URL(url).openConnection();
			//#else
			connection = (HttpURLConnection) new URL(url).openConnection();
			//#endif
			

			if (httpChunkedStreamingMode)
			{
				connection.setChunkedStreamingMode(m_ChunkedStreamSize);
			}
			
			connection.setConnectTimeout(timeOut);
			connection.setReadTimeout(readTimeout);
			
			if (httpMethod == HttpMethod.HTTP_METHOD_GET)
			{
				connection.setRequestMethod("GET");
			}
			else if(httpMethod == HttpMethod.HTTP_METHOD_POST)
			{
				connection.setRequestMethod("POST");
				connection.setDoOutput(true);	
			}
			
			connection.setFollowRedirects(true);
			
			setHttpHeaders();
		
			Iterator<Entry<String, String>> headerIter = headers.entrySet().iterator(); 
			while(headerIter.hasNext())
			{
				Entry<String,String> item = headerIter.next();
				connection.addRequestProperty(item.getKey(),item.getValue());
			}
			
			connection.setDoInput(true);			
			connection.connect();
			
			//write content to outputstream
			if(httpMethod == HttpMethod.HTTP_METHOD_POST)
			{
				OutputStream outputStream = connection.getOutputStream();
				String msg = encode();

				if (msg != null && msg.length() > 0)
				{
					byte[] content = msg.getBytes("UTF-8");				
					
					if (outputStream != null)
					{
						if (gzipUploadStream)
						{
							ByteArrayOutputStream baos = new ByteArrayOutputStream();
							outputStream = new GZIPOutputStream(baos, 9);						
						}
						
						outputStream.write(content);						
					}
				}
				
				streamContentOut(outputStream);				
				outputStream.close();
			}
						
			int statusCode = connection.getResponseCode();			
			if (statusCode != HttpURLConnection.HTTP_OK) {
				String errorMsg = connection.getResponseMessage();
				httpResponse = new HttpResponseBase(HttpResponseBase.ERROR, statusCode, errorMsg);
			} else {
				Iterator<Entry<String, List<String>>> iter = connection.getHeaderFields().entrySet().iterator();
				Hashtable<String, String> responseHeader = new Hashtable<String, String> ();
				while(iter.hasNext())
				{
					Entry<String, List<String>> item = iter.next();
					String key = item.getKey();
					List<String> values = item.getValue();
					if (key != null && key.length() > 0 & !values.isEmpty())
					{
						String value = values.get(0);
						if (value != null && value.length() > 0)						
							responseHeader.put(key, value);						
					}
				}
				
				inputStream = connection.getInputStream();
                String encoding = connection.getContentEncoding();
                if (encoding != null){
                	encoding = encoding.toLowerCase();
                	if (encoding.indexOf("gzip") >= 0)
                		inputStream = new GZIPInputStream(inputStream);
                }
               
				httpResponse = decode(inputStream, responseHeader);
			}				
		}
		catch(IOException e) {
			cleanupErrorStream = true;
			httpResponse = new HttpResponseBase(HttpResponseBase.ERROR, HttpResponseBase.NO_ERROR_CODE, e.toString());
		}
		catch (Exception e) {
			httpResponse = new HttpResponseBase(HttpResponseBase.ERROR, HttpResponseBase.NO_ERROR_CODE, e.toString());
		}
		finally
		{
			// NOTE v-yurych: Since reading is done in onResult(), for correctness it needs to specify that 
			// it throws IOException to indicate an error during the reading.  
			// We would catch the exception and deal with the error stream.
			// But because error case is a rare scenario, we don't deal with it for now, meaning
			// that the connection with error will not be re-used, 
			// but eventually will be closed by the garbage collector.
			if (httpCallback != null && !cancelled)
			{
				try
				{
					//Here if we respond too quick (usually in error case), UI has no time to handle the message. For example,
					//toast message has not been started. So we need a delay.
					if (httpResponse.getStatusCode() != HttpResponseBase.SUCCESS && System.currentTimeMillis() - requestSentoutTimeStamp < 1000) //delay 1 second.
					{
						Thread.sleep(1000);
					}

					httpCallback.onResult(httpResponse);
				}
				catch(Exception e){}
				// other exception after http stream is back is possible, for
				// example, SocketException when we have bigger downloads.
			}		
			
			if(connection != null) {
				
				
				finalCleanup(connection);
	        	// free up network resources for connection re-use
				try {
					if(cleanupErrorStream) {
						final InputStream errorStream = connection.getErrorStream();
						if(errorStream != null) {
							consumeStream(errorStream);
							errorStream.close();
						}
					}
					else {
						if(inputStream != null) {
							inputStream.close();
						}
					}
				}
				catch (Exception e) {
					Log.e("HttpRequest", e.getMessage());
				}
			}
		}
	}
	
	//Final chance to do cleanup. voice connection will call connect.disconnect.
	protected void finalCleanup(HttpURLConnection connection)
	{		
	}
	
	private static void consumeStream(final InputStream stream) throws IOException {
		byte[] buf = new byte[1024];
		while(stream.read(buf) >= 0)
			;
	}
}
