package com.microsoft.mobileexperiences.bing.httpthreadpool;

import java.io.ByteArrayInputStream;
import java.util.Hashtable;

import android.util.Log;

public class NativeHttpRequest extends HttpRequest implements IHttpResult
{
    // Holding address of the native callback.
    private long callback;
    private boolean cacheable = true;
    private static int maxRetries = 3;
    private int retry = 0;
    private Object lock = new Object();
    
    public NativeHttpRequest(String url, HttpMethod httpMethod,
            Priority priority, boolean cacheable, long callback) 
    {
        super(url, httpMethod, priority);
        this.callback = callback;
        this.cacheable = cacheable;
    }
    
    public NativeHttpRequest(String url, long callback) 
    {
        this(url, HttpMethod.HTTP_METHOD_GET, Priority.HIGH, true, callback);
    }
    
    public boolean getCacheable()
    {
        return cacheable;
    }
			
    public void setCacheable(boolean isCacheable)
    {
        cacheable = isCacheable;
    }
    
    //@Override
    public void run()
    {
        if (getCacheable())
        {
            // TODO: Use the cache supplied by shell.
            byte[] content = FileDBCacheHelper.instance().getFile(url);
            if(content != null)
            {
                ByteArrayInputStream inputStream = new ByteArrayInputStream(content); 
                Hashtable<String, String> headers = new Hashtable<String, String>();
                headers.put("Cached", "Yes");
                HttpDefaultResponse httpResponse = new HttpDefaultResponse(inputStream, headers);
                httpResponse.setStatusCode(HttpResponseBase.SUCCESS);
                if (httpCallback != null && !cancelled)
                {
                    httpCallback.onResult(httpResponse);
                }
                return;
            }
        }
        super.run();
    }

    //@Override
    public void onResult(HttpResponseBase response)
    {
        synchronized (this.lock)
        {
          if (!super.cancelled)
          {
            //Android bug: sometimes consecutive connection gets invalid response, need to retry.
            if(response.getStatusCode() == -1 && retry < maxRetries)
            {
                retry ++;
                HttpThreadPool.instance().sendRequest(this, this);
            }
            else
            {
                
                byte[] data = null;
                boolean error = true;
                boolean cached = false;
                try
                {
                    if(response.getStatusCode() == HttpResponseBase.SUCCESS)
                    {
                        HttpDefaultResponse resp = (HttpDefaultResponse)response;
                        if(resp.getHeaders().containsKey("Cached"))
                        {
                            cached = true;
                        }
                        data = HttpDefaultResponse.getResponseByteArray(resp.getResponseStream());
                        error = false;
                        //Add if it's not already in cache.
                        if (!cached)
                        {
                            Hashtable<String, String> headers = resp.getHeaders();
                            long expiration = parseCacheControl(headers.get("Cache-Control"));
                            if (expiration != 0)
                            {
                                FileDBCacheHelper.instance().insertFile(this.url, data, expiration);
                            }
                        }
                        else
                        {
                            FileDBCacheHelper.instance().updateFile(this.url);
                        }
                    }
                }
                catch (Exception e)
                {
                    Log.e("Bing", "Error on NativeHttpRequest callback " + e.getMessage());
                    data = null;
                    error = true;
                    cached = false;
                }
                nativeNetworkCallback(callback, data, error, cached);
            }
          }
        }
    }
    
    private long parseCacheControl(String cacheControl)
    {
        //Expire after 1 year, even if no expiration date is set.
        long expiration = System.currentTimeMillis() + 60*60*24*365*1000;
        if (cacheControl != null)
        {
            expiration = 0;
            String[] controls = cacheControl.toLowerCase().split("[ ,;]");
            for (int i = 0; i < controls.length; i++) 
            {
                if (controls[i].equals("no-store") || controls[i].equals("no-cache"))
                {
                    expiration = 0;
                    break;
                }
                else if (controls[i].startsWith("max-age"))
                {
                    int separator = controls[i].indexOf('=');
                    if (separator < 0) 
                    {
                        separator = controls[i].indexOf(':');
                    }
                    if (separator > 0) 
                    {
                        String s = controls[i].substring(separator + 1);
                        try 
                        {
                            long second = Long.parseLong(s);
                            if (second >= 0) 
                            {
                                expiration = System.currentTimeMillis() + 1000 * second;
                                break;
                            }
                        }
                        catch (NumberFormatException ex) 
                        {
                            expiration = 0;
                            break;
                        }
                    }
                }
            }
        }
        return expiration;
    }

    @Override
    public void setHttpHeaders() 
    {
    	// work-around for HTTP 406 response from some servers
    	addHttpHeader("Accept:","*/*");
    }

    @Override
    public void cancel()
    {
        synchronized (this.lock)
        {
          super.cancel();
        }
    }
    
    public static native void nativeNetworkCallback(long callback, byte[] data, boolean error, boolean cached);
}
